# -*- coding: utf-8 -*-

from django.db import models

from ..models import ScopedModel


class Article(ScopedModel):

    title = models.CharField(max_length=255)
    published = models.BooleanField(default=False)
    has_comments = models.BooleanField(default=False)

    def __unicode__(self):
        return self.title

    class Scopes:

        @staticmethod
        def published(qs):
            return qs.filter(published=True)

        @staticmethod
        def drafts(qs):
            return qs.filter(published=False)

        @staticmethod
        def with_comments(qs):
            return qs.filter(has_comments=True)

    class Meta:
        ordering = ("pk", )


class Comment(ScopedModel):

    article = models.ForeignKey(Article)
    title = models.CharField(max_length=255)
    published = models.BooleanField(default=False)

    class Scopes:

        @staticmethod
        def published(qs):
            return qs.filter(published=True)


class User(ScopedModel):

    is_admin = models.BooleanField(default=False)
    liked = models.ManyToManyField(Article)

    class Scopes:
        
        @staticmethod
        def admin(qs):
            return qs.filter(is_admin=True)




# -*- coding: utf-8 -*-

import functools

from django.db import models
from django.contrib.auth.models import AbstractBaseUser


__all__ = ("ScopedModel", "ScopedManager", "ScopedQuerySet",
           "ScopedBase", )


class ScopedQuerySet(models.query.QuerySet):

    def first(self):
        lst = list(self[:1])
        return lst[0] if lst else None

    def __getattr__(self, attr, *args):
        try:
            super(ScopedQuerySet, self).__getattr__(attr, *args)
        except AttributeError:
            method = getattr(self.model.Scopes, attr)
            curried = functools.partial(method, self)
            functools.update_wrapper(curried, method)
            return curried


class ScopedManager(models.Manager):

    use_for_related_fields = True

    def get_query_set(self):
        return ScopedQuerySet(self.model)

    def __getattr__(self, attr, *args):
        if attr.startswith("__"):
            raise AttributeError
        try:
            return super(ScopedManager, self).__getattr__(attr, *args)
        except AttributeError:
            return getattr(self.get_query_set(), attr, *args)


class ScopedBase(models.base.ModelBase):

    def __new__(cls, name, bases, dct):
        if name != "ScopedModel":
            if bases[0] is ScopedModel and "objects" not in dct:
                dct.update(objects=ScopedManager())
        if "Scopes" not in dct:
            raise RuntimeError("%s must have Scopes declared" % name)
        return models.base.ModelBase.__new__(cls, name, bases, dct)


class ScopedModel(models.Model):

    __metaclass__ = ScopedBase

    class Meta:
        abstract = True

    class Scopes:
        pass





# -*- coding: utf-8 -*-

from django.utils import unittest

from .models import Article, User


class ScopedTestCase(unittest.TestCase):

    def setUp(self):
        
        self.published = Article.objects.create(title="published",
                                                published=True)
        
        self.draft = Article.objects.create(title="draft",
                                            published=False)

        self.with_comments = Article.objects.create(title="with_comments",
                                                    published=True,
                                                    has_comments=True)

        self.with_comments.comment_set.create(title="published",
                                              published=True)

        self.with_comments.comment_set.create(title="published",
                                              published=False)

        self.user = User.objects.create()
        self.user.liked.add(self.published)
        self.user.liked.add(self.with_comments)


    def tearDown(self):

        Article.objects.all().delete()


    def test_scoped(self):
        
        self.assertEqual(self.published.pk,
                         Article.objects.published().first().pk)

        self.assertEqual(self.draft.pk,
                        Article.objects.drafts().first().pk)


    def test_chaining(self):
        
        self.assertEqual(self.with_comments.pk,
                         Article.objects.published().with_comments().first().pk)

        self.assertEqual(None,
                         Article.objects.drafts().with_comments().first())


    def test_related(self):

        self.assertEqual(1,
                         self.with_comments.comment_set.published().count())

        self.assertEqual(2,
                         self.user.liked.count())

        self.assertEqual(1,
                         self.user.liked.with_comments().count())

        self.assertEqual(1,
                         self.published.user_set.count())

        self.assertEqual(0,
                         self.published.user_set.admin().count())



# -*- coding: utf-8 -*

from setuptools import setup

setup(
    name="scoped",
    version="0.1.1",
    packages=["scoped", "scoped.tests"],
    install_requires=[
        "Django>=1.5",
    ],
    author="Viktor Kotseruba",
    author_email="barbuzaster@gmail.com",
    description="easily add scopes to django models",
    license="MIT",
    keywords="django",
    zip_safe=True
)
